;*******************************************************************************
;*
;*      Titel:	Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
;*              Maus-Emulation
;*              (usb_small_lib_hid_mouse_firmware_v1_3_a.asm)
;*              
;*	Datum:	11/03/12	Version:	1.3 (07/04/16)
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*      Controller: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
;*                 (siehe Zuweisungen weiter unten)
;*      Quarz:     8 oder 16 MHz  (siehe Zuweisungen weiter unten)
;*      
;*      Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
;*     
;*      Eingaenge:   PB7 (Joysticktaste), PB0 (Joystick Links)
;*                   PB1 (Joystick Rauf), PB2 (Joystick Rechts)
;*                   PB3 (Joystick Runter)
;*      Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS
;*      
;*      AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
;*
;*      Eingaenge:   PB5 (Joysticktaste), PB6 (Joystick Links)
;*                   PB7 (Joystick Rauf), PE4 (Joystick Rechts)
;*                   PE5 (Joystick Runter)
;*      Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
;*      
;*      Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
;*
;*      Eingaenge:   PB7 (Joysticktaste), PB0 (Joystick Links)
;*                   PB1 (Joystick Rauf), PB2 (Joystick Rechts)
;*                   PB3 (Joystick Runter)
;*      Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS
;*                  
;*      Informationen zur Funktionsweise: 
;*      
;*      Hauptprogramm zur Nutzung der HID-Bibliothek
;*      Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
;*      Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
;*      abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
;*      USB-Bibliothek.
;*      
;*      Fuer beide Endpunkte steht ein 64-Byte großer Buffer zur Verfuegung. 
;*      
;*      Beim Endpunkt 1 (IN) fragt der PC Daten an.
;*      Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
;*      neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
;*      Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP1_BUF) abgelegt.
;*      
;*      Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
;*      Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
;*      Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie 
;*      viele Bytes gesendet wurden.
;*      
;*      Das Freischalten der Maus-Emulation erfolgt durch die Zeile
;*      .EQU MOUSE = 1
;*     
;*      Weitere Dokumentation auf www.weigu.lu/b/usb
;*      
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*      
;*      This program is free software: you can redistribute it and/or modify
;*      it under the terms of the GNU General Public License as published by
;*      the Free Software Foundation, either version 3 of the License, or
;*      (at your option) any later version.
;*
;*      This program is distributed in the hope that it will be useful,
;*      but WITHOUT ANY WARRANTY; without even the implied warranty of
;*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*      GNU General Public License for more details.
;*
;*      You should have received a copy of the GNU General Public License
;*      along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;*      Licence: GPL-3
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
;       Einbinden der controllerspez.Definitionsdatei und Auswahl des Boards
;-------------------------------------------------------------------------------
.NOLIST                         ;List-Output ausschalten

;Hier bitte auswaehlen welches Board benutzt wir!!
;Teensy 2.0: ATmega32u4  16 MHz
;AT90USBKEY: AT90USB1287  8 MHz
;Teensy 1.0: AT90USB162  16 MHz

.EQU TEENSY2    = 1             ;eines der Flags auf 1, die anderen auf 0
.EQU AT90USBKEY = 0
.EQU TEENSY1    = 0

.IF TEENSY2 == 1
.INCLUDE "m32u4def.inc"         ;AVR-Definitionsdatei einbinden
.EQU CRYSTAL = 16
.ENDIF

.IF AT90USBKEY == 1
.INCLUDE "usb1287def.inc"
.EQU CRYSTAL = 8
.ENDIF

.IF TEENSY1 == 1
.INCLUDE "usb162def.inc"
.EQU CRYSTAL = 16
.ENDIF

.LIST                           ;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
.ORG    SRAM_START

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG    0x0000                  ;Programm beginnt an der FLASH-Adresse 0x0000
RESET1: rjmp    INIT            ;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;       Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    USB_GENaddr             ;interner Vektor fuer USB General Interrupt
        rjmp    ISRGEN          ;Springe zur ISR des GENeral Interrupt
.ORG    USB_COMaddr             ;interner Vektor fuer USB Endpoint/Pipe
                                ;COMmunication Interrupt
        rjmp    ISRCOM          ;Springe zur ISR des EP COMmunication Interrupt
.ORG    OC1Aaddr                ;Timer/Counter1 Compare Match A
        rjmp    ISRTC1          ;springe zur ISR Timer1

;-------------------------------------------------------------------------------
;       Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero  = r15             ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF    Tmp1  = r16             ;Register 16 dient als erster Zwischenspeicher
.DEF    Tmp2  = r17             ;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1  = r18             ;Register 18 dient als Zaehler
.DEF    WL    = r24             ;Register 24 und 25 dienen als universelles
.DEF    WH    = r25             ;Doppelregister W und zur Parameteruebergabe
.DEF    W     = r24

;uncomment if not needed
;falls nicht erwünscht, auskommentieren 
.EQU MOUSE = 1
;.EQU KEYBOARD = 1

        ;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
        ldi     Tmp1,HIGH(RAMEND)       ;RAMEND (SRAM) ist in der Definitions-
        out     SPH,Tmp1                ;datei festgelegt
        ldi     Tmp1,LOW(RAMEND)
        out     SPL,Tmp1

        ;Im Originalzustand ist das Fusebit CKDIV8 gesetzt, was eine Teilung
        ;durch 8 der Frequenz bewirkt
        ldi     Tmp1,0x80        ;CLKPCE = 1 Wirkung des Fusebit CKDIV
        sts     CLKPR,Tmp1       ;ueberschreiben
        clr     Tmp1             ;CLKPR = 0, Teiler = 1
        sts     CLKPR,Tmp1

        ;Timer 1 initialisieren
        ;Interrupt zwei mal pro Sekunde
        ;Vorteiler 1024, Zaehlschritte 7812-1 ergeben bei 16 MHz 1 Sekunde
        clr     Tmp1
        sts     TCCR1A,Tmp1
        ldi     Tmp1,0b00001101 ;WGM12 = 1 Teiler = 1024
        sts     TCCR1B,Tmp1
.IF CRYSTAL == 8
        ldi     Tmp1,HIGH(3905) ;8MHz
        ldi     Tmp2,LOW(3905)  ;16MHz
.ELSE        
        ldi     Tmp1,HIGH(7811)
        ldi     Tmp2,LOW(7811)
.ENDIF        
        sts     OCR1AH,Tmp1     ;Achtung High zuerst!
        sts     OCR1AL,Tmp2
        ldi     Tmp1,0x02       ;OCI1A Interrupt erlauben
        sts     TIMSK1,Tmp1
        
.IF AT90USBKEY == 1
        sbi     DDRD,4          ;PD4 rot   4 LEDs on Board 
        sbi     DDRD,5          ;PD5 gruen
        sbi     DDRD,6          ;PD6 gruen
        sbi     DDRD,7          ;PD7 rot
        cbi     DDRB,5          ;Eingang (Joysticktaste (Select))
        cbi     DDRB,6          ;Eingang (Joystick Links)
        cbi     DDRB,7          ;Eingang (Joystick Rauf)
        cbi     DDRE,4          ;Eingang (Joystick Rechts)
        cbi     DDRE,5          ;Eingang (Joystick Runter)
        sbi     PORTB,5         ;Pull-Ups
        sbi     PORTB,6          
        sbi     PORTB,7          
        sbi     PORTE,4          
        sbi     PORTE,5          
.EQU    BUTT_P  = PINB          ;Linke Maustaste
.EQU    BUTT_N  = 5    
.EQU    XL_P    = PINB          ;Links
.EQU    XL_N    = 6
.EQU    YL_P    = PINB          ;Rauf
.EQU    YL_N    = 7
.EQU    XR_P    = PINE          ;Rechts
.EQU    XR_N    = 4
.EQU    YR_P    = PINE          ;Runter
.EQU    YR_N    = 5

.ELSE
        sbi     DDRD,6          ;PD6 Blink LED   Teensy2 on Board
        cbi     DDRB,7          ;Eingang (Joysticktaste (Select))
        cbi     DDRB,0          ;Eingang (Joystick Links)
        cbi     DDRB,1          ;Eingang (Joystick Rauf)
        cbi     DDRB,2          ;Eingang (Joystick Rechts)
        cbi     DDRB,3          ;Eingang (Joystick Runter)
        sbi     PORTB,7         ;Pull-Ups
        sbi     PORTB,0          
        sbi     PORTB,1          
        sbi     PORTB,2          
        sbi     PORTB,3          
.EQU    BUTT_P  = PINB          ;Linke Maustaste
.EQU    BUTT_N  = 7    
.EQU    XL_P    = PINB          ;Links
.EQU    XL_N    = 0
.EQU    YL_P    = PINB          ;Rauf
.EQU    YL_N    = 1
.EQU    XR_P    = PINB          ;Rechts
.EQU    XR_N    = 2
.EQU    YR_P    = PINB          ;Runter
.EQU    YR_N    = 3
        
.ENDIF

        rcall   USBINI          ;befindet sich in der USB-Bibliothek
;-------------------------------------------------------------------------------
;        Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:   ldi     Tmp1,4          ;Maus liefert 4 Byte!
        sts     EP1_CNT,Tmp1
        ;Linke Maustaste
        sbic    BUTT_P,BUTT_N
        rjmp    MB1
        ldi     Tmp1,1          ;Falls Button Gedrueckt(nullak.)
        sts     Ep1_buf,Tmp1    ;dann setze Butt1 (Linke Maustaste)
        rjmp    X_R
MB1:    sts     Ep1_buf,Zero
X_R:    sbic    XR_P,XR_N       ;X-Koordinate
        rjmp    X_L
        ldi     Tmp1,1          ;Falls Rechts gedrueckt X = X+1
        sts     Ep1_buf+1,Tmp1
        rjmp    Y_R
X_L:    sbic    XL_P,XL_N
        rjmp    X_0
        ldi     Tmp1,0xFF       ;Falls Links X = X-1 (Zweierkomplement)
        sts     Ep1_buf+1,Tmp1
        rjmp    Y_R
X_0:    sts     Ep1_buf+1,Zero
Y_R:    sbic    YR_P,YR_N       ;Y-Koordinate
        rjmp    Y_L
        ldi     Tmp1,1          ;Falls Runter gedrueckt Y = Y+1
        sts     Ep1_buf+2,Tmp1
        rjmp    SCR
Y_L:    sbic    YL_P,YL_N
        rjmp    Y_0
        ldi     Tmp1,0xFF       ;Falls Rauf Y = Y -1
        sts     Ep1_buf+2,Tmp1
        rjmp    SCR
Y_0:    sts     Ep1_buf+2,Zero
SCR:    sts     Ep1_buf+3,Zero
        ldi     Tmp1,1          ;EP1 Flag setzen weil Daten vorhanden
        sts     EP1_FLAG,Tmp1
        
        rjmp    MAIN            ;Endlosschleife

;-------------------------------------------------------------------------------
;       Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
;Timer 1 ISR Interrupt jede Sekunde
ISRTC1: push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16

        sbi     PIND,6          ;Toggle LED

        pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti            

.INCLUDE "lib/SR_USB_HID_V1_3.asm" ;USB Bibliothek

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT                           ;Ende des Quelltextes
