' small usb library for AVRs (mega32u4, mega32u2,at90usb162, at90usb1287)
' using hid class
' 2 Endpoints(1 * In 1 * Out) With 64 Byte
'
' Kleine Usb -Bibliothek:
' Unterprogramme und Interruptroutinen zur Nutzung von USB mit
' ATMEL-AVR-ICs.
' HID Klasse (usb_lib_hid_v1_2.bas)
'
' Datum:  18/02/13                Version: 1.2 (01/06/13)
'
' Prozessor: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
' (siehe Zuweisungen im Hauptprogramm)
' Quarz: 8 oder 16 MHz  (siehe Zuweisungen im Hauptprogramm)
'
' Informationen zur Funktionsweise:
'
'    Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
'    abgewickelt. Alle dazu benoetigten USB-Routinen und Unterprogramme
'    befinden sich in dieser USB-Bibliothek.
'
'    Im Hauptprogramm muss gleich am Anfang diese Bibliothek eingebunden werden,
'    damit globale Variablen eingebunden werden. Leider muss sie mit einem Goto
'    uebersprungen werden, da sonst der Code der Isr ausgefuehrt wuerde.
'
'    Goto Init
'    $include "/lib/usb_lib_hid_v1_2.bas"    'USB-Bibliothek einbinden
'    Init:

'    Kurz vor der Hauptschleife soll dann das Unterprogramm Usb_init_device
'    aufgerufen werden, das auch die Interrupts global freischaltet.
'
'    Fuer beide Endpunkte steht ein 64-Byte grosser Buffer zur Verfuegung.
'
'    Beim Endpunkt 1 (IN) fragt der PC Daten an.
'    Durch Setzen eines Flags (EP1_FLAG) im Hauptprogramm wird dem Endpunkt
'    mitgeteilt, wenn neue Daten vorliegen. Die Anzahl der vorhandenen Bytes
'    steht dann in der Variablen EP1_CNT. Die Daten selbst werden im Buffer
'    (EP1_BUF) abgelegt.

'    Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
'    Die Hauptschleife im Hauptprogramm erkennt an einem Flag (EP2_FLAG)
'    wenn Daten vorliegen.
'    Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie
'    viele Bytes gesendet wurden.
'
'    Im Haptprogramm muessen die Konstanten Mouse und Keyboard definiert sein.
'    Mit ihnen kann ausgewaehlt werden ob eine Computermaus emuliert werden soll
'    (MOUSE = 1) oder eine Tastatur (Keyboard = 1).
'
'    Weitere Dokumentation auf www.weigu.lu/b/usb
'
'
' Copyright (c) 2013   Jean-Claude FELTES & Guy WEILER
'                                           weigu[at]weigu[dot]lu

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' Licence: GPL-3


'**********************   USB-Routinen     ************************************

'Teensy 1.0: AT90USB162  16 MHz
'Teensy 2.0: ATmega32u4  16 MHz
'AT90USBKEY: AT90USB1287  8 MHz

#if _chip = 75                          'mega32u4
'Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)
'PPL benoetigt 8 MHz
 #if _xtal = 8000000
  Const Pllpre = &H00                   'Bit 2-4 = 0b000 (:1)
 #else
  Const Pllpre = &H10                   'Bit 2-4 = 0b100 (:2)
 #endif
#endif

#if _chip = 78 Or _chip = 74            'mega32u2 oder 90usb162
'Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)
'PPL benoetigt 8 MHz
 #if _xtal = 8000000
  Const Pllpre = &H00                   'Bit 2-4 = 0b000 (:1)
 #else
  Const Pllpre = &H04                   'Bit 2-4 = 0b001 (:2)
 #endif
#endif

#if _chip = 70                          '90usb1287
'Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)
'PPL benoetigt 2 MHz
 #if _xtal = 8000000
  Const Pllpre = &H0C                   'Bit 2-4 = 0b011 (:4)
 #else
  Const Pllpre = &H14                   'Bit 2-4 = 0b101 (:8)
 #endif
#endif

'USB-Variablen fr Init_Endpoint

Dim Ep_num As Byte                      'Nummer des anzusprechenden EndPoints
Dim Ep_typ As Byte                      'Typ  (0=Contr., 1=Iso, 2=Bulk, 3=Interrupt)
Dim Ep_dir As Bit                       'bertragungsrichtung (0=OUT-EP, 1=IN-EP)
Dim Ep_siz As Byte                      'Anzahl codiert als 2^(Ep_siz+3)
Dim Ep_bk As Byte                       'EP-Bank (0=1 Bank, 1=Banks)

'fr SETUP Paket USB
Dim Su_breq_type As Byte                'bmRequestType
Dim Su_breq As Byte                     'bRequest
Dim Su_val As Word                      'wValue
Dim Su_val_l As Byte At Su_val Overlay
Dim Su_val_h As Byte At Su_val + 1 Overlay
Dim Su_ind As Word                      'wIndex
Dim Su_ind_l As Byte At Su_ind Overlay
Dim Su_ind_h As Byte At Su_ind + 1 Overlay
Dim Su_len As Word                      'wLength
Dim Su_len_l As Byte At Su_len Overlay
Dim Su_len_h As Byte At Su_len + 1 Overlay

'Zwischenvariablen fr USB-Routinen
Dim Btmp As Byte
Dim Btmp2 As Byte
Dim Bctr As Byte
Dim I As Byte
Dim I_USB As Byte

'Buffer fuer die Endpunkte
Dim Ep1_buf(64) As Byte                 'Buffer fuer zu sendende Daten (zum PC)
Dim Ep2_buf(64) As Byte                 'Buffer fuer ankommende Daten (vom PC)

'Flags und Zaehler fuer die Endpunkte
Dim Ep1_flag As Bit                     'Flag Senden zum PC
Dim Ep2_flag As Bit                     'Flag Empfang vom PC, wird automatisch gesetzt
                                                             'falls Daten vorhanden
Dim Ep1_cnt As Byte                     'Anzahl der Datenbytes Senden zum PC
Dim Ep2_cnt As Byte                     'Anzahl der Datenbytes Empfang vom PC

'USB-Konstanten fr Endpoints
Const Ep0_t = 0                         'EP0-Typ: 0=Control
Const Ep0_d = 0                         'EP0-DIR:0=OUT
Const Ep0_s = 0                         'EP0-Size 2^(0+3)=8 Byte
Const Ep0_b = 0                         '0 = 1 Bank
Const Ep0_fs = 8                        'FIFO size = 8 Byte (Word)

'EP1 = IN EP (PC <- Device)
Const Ep1_t = 3                         'EP1-Typ: Interrupt Transfer
Const Ep1_d = 1                         'EP1-DIR: 1=IN  (zum PC)
Const Ep1_s = 3                         'EP1-Size 2^(3+3)=64 Byte
Const Ep1_b = 0                         '0 = 1 Bank
Const Ep1_fs_l = 64                     'FIFO size = 64 Byte (Word)
Const Ep1_fs_h = 0

'EP2 = OUT EP (PC -> Device)
Const Ep2_t = 3                         'EP2-Typ: Interrupt Transfer
Const Ep2_d = 0                         'EP2-DIR: 0=OUT (vom PC)
Const Ep2_s = 3                         'EP2-Size 2^(3+3)=64Byte
Const Ep2_b = 0                         '0 = 1 Bank
Const Ep2_fs_l = 64                     'FIFO size = 64 Byte (Word)
Const Ep2_fs_h = 0

'''EP... = ...
'''Hier erweitern


'weitere globale Konstanten
'bei Erweiterung folgende beiden Konstanten anpassen
Const Nr_eps = 2                        'Zahl der benutzten EP (ohne EP0)
'Annahme, dass wTotallength kleiner als 256 Byte ist!
Const Totallength = &H09 + &H09 + &H09 +(&H07 * Nr_eps)

#if Mouse = 1
Const Hid_descr_length = 52
#elseif Keyboard = 1
Const Hid_descr_length = 59
#else
Const Hid_descr_length = 32
#endif

Const Lang_i = 0
Const Manu_i = 1
Const Prod_i = 2
Const Seri_i = 3

'Konstante fr Unterprogramm USB_init_device
'mega32u2 oder '90usb162 besitzen das Bit OTGPADE (enable VBUS Pad) nicht!
#if _chip = 78 Or _chip = 74            'mega32u2 oder '90usb162
Const Usbc1 = &HA0
#else
Const Usbc1 = &HB0
#endif


'-------------------------------------------------------------------------------
'          USB INTERRUPT SERVICE ROUTINEN
'-------------------------------------------------------------------------------

Usb_isr_general:
' ** USB General Interrupt Service Routine
' reagiert nur auf End Of Reset Interrupt EORSTI
If Udint.eorsti = 1 Then
    'General Interrupt sperren
    Udint.eorsti = 0                    'lsche End Of Interrupt Flag
    'Initialisierung  EP0
    Ep_num = 0
    Ep_typ = Ep0_t
    Ep_dir = Ep0_d
    Ep_siz = Ep0_s
    Ep_bk = Ep0_b
    Gosub Usb_init_endpoint
    'erlaube EP COM Interrupts
    Ueienx.rxstpe = 1
End If
Return
'------------------------------------------------------------------------------

Usb_isr_communication:
'** Endpoint/Pipe Communication Interrupt Service Routine

'EP-Nummer durch Bitposition in UEINT  ermitteln
    Select Case Ueint
        Case 1
            'EP0: Enumeration ber EP0
            Uenum = 0                   'EP0  auswhlen
            'Wenn Setup-Paket von PC angekommen, dieses behandeln
            If Ueintx.rxstpi = 1 Then Gosub Usb_ep0_setup
        Case 2
            'EP1: Werte an PC bergeben (IN EP)
            Uenum = 1                   'EP1  auswhlen
            Gosub Usb_ep1_in
        Case 4
            'Ep2: Informationen vom PC verarbeiten (OUT  EP)
            Uenum = 2
            Gosub Usb_ep2_out

            '''hier erweitern
            '''Case 8
            '''EP5: UENUM = 3....

        Case Else
        'andere Flle (die hier nicht implementiert sind)
    End Select
Return
'-------------------------------------------------------------------------------

'-------------------------------------------------------------------------------
'          USB Unterprogramme
'-------------------------------------------------------------------------------

Usb_init_device:
'USB-Aktivierung (Full Speed 12Mbit/s, Interrupts erlauben)

#if _chip = 75                          'mega32u4
    Uhwcon = &H01                       'Versorgungsspannung D+, D- ein   (''')
#endif
#if _chip = 70                          '90usb1287
    Usbcon.frzclk = 1                   'Der ATMEL Bootloader nutzt einige Interrupts
    Otgien = 0                          'Deaktiviere diese Interrupts damit sauberer Start
    Udien = 0
    Uhien = 0
    Ueienx = 0
    Upienx = 0
    Uhwcon = &H80                       'Enable Device Modus
    Uhwcon = &H81                       'Regelkreis zur Versorgung Versorgungsspannung D+, D- ein   (''')
#endif

    Usbcon = Usbc1                      'Sender/Empf., Takt einschalten, VBUS einschalten
    Usbcon.frzclk = 0                   'FRZCLK=0 ntig (kurz), da sonst kein WAKEUP Interrupt
    Usbcon.frzclk = 1                   'FRZCLK=1, Strom sparen

    'PLL starten
    Pllcsr = Pllpre                     'Vorteiler fr Quarz (siehe Zuweisungen
    Pllcsr.plle = 1                     'starte PLL (Enable-Bit = 1)
    Bitwait Pllcsr.plock , Set          'warten bis PLL eingerastet (ca. 100ms)
    Usbcon.frzclk = 0                   'FRZCLK=0: aktiviere Takt

    Udcon.detach = 0                    'Usb attach connect the device
    Udien.eorste = 1                    'Usb enable End of Reset Interrupt (EORSTE=1)

    'Flags, Zhler und Buffer lschen
    Ep1_flag = 0
    Ep2_flag = 0
    Ep1_cnt = 0
    Ep2_cnt = 0

    Dim Index As Integer
    For Index = 1 To 64
        Ep1_buf(index) = 0
    Next Index
    For Index = 1 To 64
         Ep2_buf(index) = 0
    Next Index

  Enable Interrupts
Return
'-------------------------------------------------------------------------------
Usb_init_endpoint:
'Endpoint initialisieren
'Ep_num, Ep_typ, Ep_dir, Ep_siz, Ep_bk mssen vorher gesetzt sein!

    Uenum = Ep_num                      'EP auswhlen
    Ueconx.epen = 1                     'und aktivieren
    Uecfg0x.0 = Ep_dir                  'Datenrichtung
    Uecfg0x.6 = Ep_typ.0                'Typ in 2 Bits
    Uecfg0x.7 = Ep_typ.1                'Typ in 2 Bits
    'Uecfg1x.alloc = 0
    Uecfg1x.2 = Ep_bk.0                 'FIFO-Bank in 2 Bits
    Uecfg1x.3 = Ep_bk.1                 'FIFO-Bank in 2 Bits
    Uecfg1x.4 = Ep_siz.0
    Uecfg1x.5 = Ep_siz.1                'FIFO-Grsse in 2 Bits
    Uecfg1x.6 = Ep_siz.2                'FIFO-Grsse in 2 Bits
    Uecfg1x.alloc = 1                   'Speicher dem Puffer zuweisen
Return
'------------------------------------------------------------------------------
Usb_ep0_setup:
'Behandle eine SETUP-Anfrage vom PC
'lies Setup-Paket (8 byte) vom PC ins RAM
    Su_breq_type = Uedatx               'bmRequestType
    Su_breq = Uedatx                    'bRequest
    Su_val_l = Uedatx                   'Setup-Value in Word
    Su_val_h = Uedatx
    Su_ind_l = Uedatx                   'Setup-Index in Word
    Su_ind_h = Uedatx
    Su_len_l = Uedatx                   'Setup-Length in Word
    Su_len_h = Uedatx
    'ACK SU-Paket (ISR abschliessen und FIFO lschen)
    Ueintx.rxstpi = 0
    'Ist es ein Standard Device Request? wenn nicht: Stall und raus
    Btmp = Su_breq_type And &H61        'nur Device oder Interface
    If Btmp = 0 Or Btmp = 1 Then
        'Welche Art Request?
        Select Case Su_breq
            Case 0                      'Get status
                'nur ntig um Programme zufrieden zu stellen die den Status abfragen
                '00-Word zurcksenden (bus powered (Device Req.) bzw. 0 (Interf. Req.)
                    Uedatx = 0
                    Uedatx = 0
                    Ueintx.txini = 0    'Daten absenden (ACK) und FIFO lschen
                    'Warten bis ZLP vom PC (RXOUTI Flag gesetzt wird)
                    Bitwait Ueintx.rxouti , Set
                    Ueintx.rxouti = 0   'Flag wieder lschen
            Case 5 :                    'set Address (nur 2 Abschnitte (keine Daten))
                'Gerteadresse (0...127) steht in Low Byte von Su_val
                Udaddr = Su_val_l And &H7F
                Ueintx.txini = 0        'ZLP senden!
                'Warten bis Speicherbank wieder frei
                Bitwait Ueintx.txini , Set
                'Adresse aktivieren durch Setzen von Bit 7
                Udaddr.adden = 1

            Case 6 :                    'get descriptor
                Select Case Su_val_h
                    Case 1
                        'Device descriptor
                        Restore Devicedescriptor
                        Read Bctr
                        Restore Devicedescriptor

                    Case 2
                        'Configuration descriptor
                        Bctr = Su_len_l
                        'Manchmal erfragt Windows unsinnig hohe Werte (groesser als 256
                        'Byte). Dadurch steht dann im Lowbyte ev. ein falscher Wert
                        'Es wird hier davon ausgegangen, dass nicht mehr als 256 Byte
                        'angefragt werden knnen.
                        If Su_len > Totallength Or Su_len = 0 Then Bctr = Totallength
                        Restore Configurationdescriptor

                    Case 3
                        'String descriptor
                        Btmp = Su_val_l
                        Select Case Btmp
                            Case Lang_i
                                Restore Stringdescriptor0
                                Read Bctr
                                Restore Stringdescriptor0
                            Case Manu_i
                                Restore Stringdescriptor1
                                Read Bctr
                                Restore Stringdescriptor1
                            Case Prod_i
                                Restore Stringdescriptor2
                                Read Bctr
                                Restore Stringdescriptor2
                            Case Seri_i
                                Restore Stringdescriptor3
                                Read Bctr
                                Restore Stringdescriptor3
                        End Select
                    Case 34
                        'Report descriptor 0x22
                        Bctr = Su_len_l
                        If Su_len > Hid_descr_length Or Su_len = 0 Then
                            Bctr = Hid_descr_length
                        End If
                        Restore Reportdescriptor
                    Case Else : Return
                End Select
                'FIFO mit Descriptor fllen und zum PC schicken
                Gosub Usb_send_descriptor

            Case 9 :                    'set configuration
                'Benutzte Endpoints abschalten
                For I = Nr_eps To 1 Step -1       'Rckwrts (siehe Speicherbelegung)
                    Uenum = I
                    Ueconx.epen = 0
                    Uecfg1x.alloc = 0
                Next I
                '----------------------------------------
                'Initialisierung der Endpunkte

                'Initialisierung Endpunkt 1 (IN-EP)
                Ep_num = 1
                Ep_typ = Ep1_t
                Ep_dir = Ep1_d
                Ep_siz = Ep1_s
                Ep_bk = Ep1_b
                Gosub Usb_init_endpoint
                 'NAK INterrupts fr IN Endpunkte erlauben
                'Int. wird von HW gesetzt wenn IN Anfrage von PC mit NAK
                ' beantwortet wurde. Dann wird ein neuer COM INT ausgelst
                Ueienx.nakine = 1

               'Initialisierung Endpunkt2  (OUT-EP)
                Ep_num = 2
                Ep_typ = Ep2_t
                Ep_dir = Ep2_d
                Ep_siz = Ep2_s
                Ep_bk = Ep2_b
                Gosub Usb_init_endpoint
                'RX OUT Interrupts fr OUT Endpunkte erlauben
                'wird von HW gesetzt wenn OUT Daten vorhanden sind
                'Dann wird ein neuer COM INT ausgelst
                Ueienx.rxoute = 1

                ''' hier erweitern
                '----------------------------------------

                Uenum = 0               'EP0
                Ueintx.txini = 0        'sende ZLP (Erfolg, lscht EP-Bank)
                'Warten bis Speicherbank wieder frei
                Bitwait Ueintx.txini , Set

            Case Else
            Ueconx.stallrq = 1          'Stall (raus)
        End Select
    Else
        Ueconx.stallrq = 1              'Stall (raus)
    End If
'    Btmp = Su_breq_type And &H60
'    If Btmp = &H20 Then
'        Select Case Su_breq
'            Case 1                      'Get Report
'                'GOSUB GET_Report  'Probleme unter Virtualbox hier nicht impl.
'                Return
'            Case Else
'            Ueconx.stallrq = 1          'Stall (raus)
'        End Select
'    Else
'        Ueconx.stallrq = 1              'Stall (raus)
'    End If

Return

'-------------------------------------------------------------------------------
Usb_send_descriptor:
'FIFO mit Descriptor fllen und zum PC schicken
    I = 0
    Do
       'Flle FIFO mit Descriptorbyte
        I = I + 1
        Read Btmp
        Uedatx = Btmp
        'immer nach 8 Bytes das Paket absenden und Speicherbank lschen
        Btmp2 = I Mod 8
        If Btmp2 = 0 Then
            Ueintx.txini = 0            'Daten absenden (ACK) und FIFO lschen
            'Warten bis Speicherbank wieder frei oder ACK vom PC (ZLP)
            Do
            Loop Until Ueintx.txini = 1 Or Ueintx.rxouti = 1
        End If
        Bctr = Bctr - 1
        'Wenn Interrupt vom PC (rxouti = 1 PC bricht ab), dann zurck
    Loop Until Ueintx.rxouti = 1 Or Bctr = 0

    'kontrolliere ob ACK(ZLP) vom PC angekommen (schon fertig)
    ' (kommt vor wenn Vielfaches des FIFO-Buffers angefragt waren)
    If Ueintx.rxouti = 1 Then
        Ueintx.rxouti = 0               'Handshake um Interrupt zu besttigen
    Else
        'es sind noch restliche Bytes abzuschicken
        Ueintx.txini = 0                'Paket absenden und Speicherbank lschen
        'Warten bis ACK(ZLP) vom PC angekommen
        Bitwait Ueintx.rxouti , Set
        Ueintx.rxouti = 0               'Handshake  um Interrupt zu besttigen
    End If
Return

'-------------------------------------------------------------------------------

Usb_ep1_in:
'Endpunkt 1 IN    PC erfragt Daten vom Mikrocontroller
If Ueintx.nakini = 1 Then               'Sind Daten vom PC angefragt worden?
    Ueintx.nakini = 0                   'NAKIN Interrupt Flag lschen
    If Ueintx.txini = 1 Then            'Speicherbank frei?
        Ueintx.txini = 0                'lscht EP-Bank
        If Ep1_flag = 1 Then
            For I = 1 To Ep1_cnt
                Uedatx = Ep1_buf(i)

            Next I
        End If
        Ep1_flag = 0
        Ep1_cnt = 0
        Ueintx.fifocon = 0              'erlaube Abschicken der Daten (FIFO freigeben)
    End If
End If
Return

'-------------------------------------------------------------------------------

Usb_ep2_out:
'Endpunkt 2 OUT       Mikrocontroller erhaelt Daten vom PC
If Ueintx.rxouti = 1 Then               'neue Daten vom PC?
    Ueintx.rxouti = 0                   'Datenpaket besttigen (ACK)
    For I_usb = 1 To 64
        If Ueintx.rwal = 1 Then         'Daten im FIFO?
            Ep2_buf(i_usb ) = Uedatx
            Ep2_cnt = I_usb
        Else
             Exit For
       End If
    Next
    Ep2_flag = 1                        'Flag setzen
    Ueintx.fifocon = 0                  'FIFO wieder freigeben
End If
Return

'hier erweitern'

'-------------------------------------------------------------------------------
Devicedescriptor:
Data 18                                 '18 byte
Data &H01                               'bDescriptortype
Data &H10 , &H01                        'Device USB 1.1
Data &H00                               'bDeviceClass
Data &H00                               'Bdevicesubclass
Data &H00                               'bDeviceProtocol
Data Ep0_fs                             'FIFO Size
Data &HEB , &H03                        'id_Vendor=Atmel Code            03EB
Data &H02 , &H00                        'id_Prod (weigu: 2 FUER HID!)    0002
'Data &HFA , &H2F                        '                                2FFA
Data &H01 , &H00                        'bcd_Device (release number) 0x0100 -> 1.00
Data Manu_i                             'Index fr stringdescriptor Hersteller
Data Prod_i                             'Index fr Stringdescriptor Produkt
Data Seri_i                             'Index fr Stringdescriptor Seriennummer
Data &H01                               'bNumConfigurations
                                        ' (Anzahl mglicher Konfigurationen)
Configurationdescriptor:
Data 9                                  'Anzahl Bytes
Data &H02                               'bDescriptortype
Data Totallength , 0                    'Lnge Descriptor + alle untergeordneten Descr.
Data &H01                               'Number of Interfaces
Data &H01                               'bConfigurationValue (<>0, sonst unkonfiguriert)
Data 0                                  'iConfiguration (kein Text)
Data &H80                               'bmAttributes  (bus powered, kein remote wakeup)
Data 250                                'MaxPower (250*2mA = 500mA)

'Interface Descriptor
Data &H09                               'bLenghth
Data &H04                               'bDescriptortype
Data 0                                  'bInterfaceNumber
Data 0                                  'bAlternateSettings
Data Nr_eps                             'Number of Endpoints
Data &H03                               'bInterfaceclass = 3 = HID
Data &H00                               'bInterfaceSubClass
Data &H00                               'bInterfaceProtocol
Data 0                                  'iInterface (kein Text)

'HID Descriptor
Data &H09                               'bLenghth
Data &H21                               'bDescriptortype = 0x21 (HID Klasse)
Data &H10 , &H01                        'bcdHID = 0x0110, HID_Spec 1.1
Data 0                                  'bCountryCode = 0 (wird hier nicht unterstuetzt)
Data &H01                               'bNumDescriptors = 1 Anzahl der untergeordneten Deskriptoren
Data &H22                               'bDescriptorType = 0x22, Report Descriptor (obligatorisch)
Data Hid_descr_length , 0               'wDescriptorLength Laenge des Report Deskriptor

'EP1 Descriptor
Data &H07                               'bLength
Data &H05                               'bDescriptortype
Data &H81                               'Endpoint address (bit7=1(IN), bit0-3=EP-Nummer)
Data &H03                               'bmAttributes   (Interrupt Transfer)
Data Ep1_fs_l , Ep1_fs_h                'FIFO Size
Data 1                                  'bInterval = 1 bei Interrupt: Latenzzeit in Millisekunden

'EP2 Descriptor
Data &H07                               'bLength
Data &H05                               'bDescriptortype
Data &H02                               'Endpoint address (bit7=0(OUT), bit0-3=EP-Nummer)
Data &H03                               'bmAttributes   (Interrupt Transfer)
Data Ep2_fs_l , Ep2_fs_h                'FIFO Size
Data 1                                  'bInterval = 1 bei Interrupt: Latenzzeit in Millisekunden

'''hier erweitern

Stringdescriptor0:
'Language String Descriptor
Data 4                                  'bLength
Data 3                                  'bDescriptortype
Data 9 , 4                              'wLANGID[0] 0409h = USA English

Stringdescriptor1:
'Manufacturer
Data 30 , 3
Data "F" , "E" , "L" , "J" , "C" , " " , "W" , "E" , "I" , "G" , "U" , "." , "L" , "U"

Stringdescriptor2:
'Product String
#if Mouse = 1
Data 20 , 3
Data "U" , "S" , "B" , "-" , "M" , "O" , "U" , "S" , "E"
#elseif Keyboard = 1
Data 26 , 3
Data "U" , "S" , "B" , "-" , "K" , "E" , "Y" , "B" , "O" , "A" , "R" , "D"
#else
Data 22 , 3
Data "H" , "I" , "D" , "-" , "D" , "E" , "V" , "I" , "C" , "E"
#endif

Stringdescriptor3:
'Serial Number
Data 12 , 3
Data "W" , "H" , "1" , "2" , "b"

'Bei alles Strings: Unicode!!!
' "X" wird als 2 Byte interpretiert!

Reportdescriptor:
#if Mouse = 1
' Reportdescriptor fuer die HID-Maus
Data &H05 , &H01                        'USAGE_PAGE (Generic Desktop)
Data &H09 , &H02                        'USAGE (mouse)
Data &HA1 , &H01                        'COLLECTION (Application)
Data &H09 , &H01                        'USAGE (Pointer)
Data &HA1 , &H00                        'COLLECTION (Physical)
Data &H05 , &H09                        'USAGE_PAGE (Buttons) Maustasten
Data &H19 , &H01                        'USAGE_MINIMUM (Button 1) mindestens eine
Data &H29 , &H03                        'USAGE_MAXIMUM (Button 3) maximal 3
Data &H15 , &H00                        'LOGICAL_MINIMUM (0)
Data &H25 , &H01                        'LOGICAL_MAXIMUM (1)
Data &H75 , &H01                        'REPORT_SIZE (1) Anzahl der Bits pro Datenel.
Data &H95 , &H03                        'REPORT_COUNT (3) Anzahl der Datenelemente
Data &H81 , &H02                        'INPUT (Data,Var,Abs) INPUT Bericht, Werte abs.
Data &H75 , &H05                        'REPORT_SIZE (5) 5 Bits (fllt Byte?)
Data &H95 , &H01                        'REPORT_COUNT (1) 1*
Data &H81 , &H01                        'INPUT (Cnst,Var,Abs) Konstantes Feld
Data &H05 , &H01                        'USAGE_PAGE (Generic Desktop) Mauszeiger
Data &H09 , &H30                        'USAGE (X) 48
Data &H09 , &H31                        'USAGE (Y) 49
Data &H09 , &H38                        'USAGE (Wheel) 56
Data &H15 , &H81                        'LOGICAL_MINIMUM (-127) -127 bis 127
Data &H25 , &H7F                        'LOGICAL_MAXIMUM (127)
Data &H75 , &H08                        'REPORT_SIZE (8) 8 Bits
Data &H95 , &H03                        'REPORT_COUNT (3) 3*8 Bits (X,Y,Wheel)
Data &H81 , &H06                        'INPUT (Data,Var,Rel) Werte relativ!
Data &HC0                               'END_COLLECTION
Data &HC0                               'END_COLLECTION

#elseif Keyboard = 1
' Reportdescriptor fuer die HID-Tastatur
Data &H05 , &H01,                       ' Usage_Page (Generic Desktop)
Data &H09 , &H06,                       ' Usage (Keyboard)
Data &HA1 , &H01,                       ' Collection (Application)
Data &H05 , &H07,                       ' Usage page (Key Codes)
Data &H19 , &HE0,                       ' Usage_Minimum (224)
Data &H29 , &HE7,                       ' Usage_Maximum (231)
Data &H15 , &H00,                       ' Logical_Minimum (0)
Data &H25 , &H01,                       ' Logical_Maximum (1)
Data &H75 , &H01,                       ' Report_Size (1)
Data &H95 , &H08,                       ' Report_Count (8)
Data &H81 , &H02,                       ' Input (Data,Var,Abs) = Modifier Byte
Data &H81 , &H01,                       ' Input (Constant) = Reserved Byte
Data &H19 , &H00,                       ' Usage_Minimum (0)
Data &H29 , &H65,                       ' Usage_Maximum (101)
Data &H15 , &H00,                       ' Logical_Minimum (0)
Data &H25 , &H65,                       ' Logical_Maximum (101)
Data &H75 , &H08,                       ' Report_Size (8)
Data &H95 , &H06,                       ' Report_Count (6)
Data &H81 , &H00,                       ' Input (Data,Array) = Keycode Bytes(6)
Data &H05 , &H08,                       ' Usage Page (LEDs)
Data &H19 , &H01,                       ' Usage_Minimum (1)
Data &H29 , &H05,                       ' Usage_Maximum (5)
Data &H15 , &H00,                       ' Logical_Minimum (0)
Data &H25 , &H01,                       ' Logical_Maximum (1)
Data &H75 , &H01,                       ' Report_Size (1)
Data &H95 , &H05,                       ' Report_Count (5)
Data &H91 , &H02,                       ' Output (Data,Var,Abs) = LEDs (5 bits)
Data &H95 , &H03,                       ' Report_Count (3)
Data &H91 , &H01,                       ' Output (Constant) = Pad (3 bits)
Data &HC0                               ' End_Collection

#else
' Reportdescriptor fuer das HID-Gert
Data &H06 , &HA0 , &HFF                 'USAGE PAGE = 0xFF00 (Vendor Defined Page 1)
Data &H09 , &H01                        'USAGE (Vendor Usage 1)
                                        'Funktion des Berichts in der Usage Page
Data &HA1 , &H01                        'COLLECTION (Application) Beginn einer Gruppe
                                        'von Eintraegen
Data &H09 , &H03                        'USAGE (Vendor Usage 1)
Data &H15 , &H00                        'LOGICAL_MINIMUM (1) Legen Bereich Werte fest, die
Data &H25 , &H40                        'LOGICAL_MAXIMUM (64) im Bericht enthalten sind (bin.)
Data &H95 , &H02                        'REPORT_COUNT (64) Anzahl der Datenelemente im Bericht
Data &H75 , &H08                        'REPORT_SIZE (8) Anzahl der Bits pro Datenelement 8Bit
                                        'Make sixty-four 8-bit fields (the next time the
                                        'parser hits an "Input", "Output", or "Feature" item)
Data &H81 , &H02                        'INPUT (Data,Var,Abs) INPUT Bericht
                                        'Instantiates input packet fields based on the above
                                        'report size, count, logical min/max, and usage.
Data &H09 , &H04                        'USAGE (Vendor Usage 1)
Data &H15 , &H00                        'LOGICAL_MINIMUM (1) Legen Bereich Werte fest, die
Data &H25 , &H40                        'LOGICAL_MAXIMUM (64) im Bericht enthalten sind (bin.)
Data &H95 , &H02                        'REPORT_COUNT (64) Anzahl der Datenelemente im Bericht
Data &H75 , &H08                        'REPORT_SIZE (8) Anzahl der Bits pro Datenelement 8Bit
Data &H91 , &H02                        'OUTPUT (Data,Var,Abs) OUTPUT Bericht
                                         'Instantiates output packet fields.  Uses same report
                                         'size and count as "Input" fields, since nothing
                                         'new/different was specified to the parser since the
                                         '"Input" item.
Data &HC0                               ' END_COLLECTION

#endif