' main source for using the small usb library using hid class (hid keyboard)
' Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
' Tastatur-Emulation
'
' Datum:  18/02/13                Version: 1.2 (28/04/13)
'
' Prozessor: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
'
' Quarz: 8 oder 16 MHz  (siehe Zuweisungen)
'
' Informationen zur Beschaltung:
'      Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
'
'      Eingaenge:   PB0-7 (8 Tasten)
'      Ausgaenge:   PD6 (LED on board)
'
'      AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
'
'      Eingaenge:   PB0-7 (8 Tasten) ev. anpassen
'      Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
'
'      Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
'
'      Eingaenge:   PB0-7 (8 Tasten)
'      Ausgaenge:   PD6 (LED on board)
'
'      Informationen zur Funktionsweise:
'
'      Hauptprogramm zur Nutzung der HID-Bibliothek
'      Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
'      Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
'      abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
'      USB-Bibliothek.
'
'      Fuer beide Endpunkte steht ein 64-Byte groer Buffer zur Verfuegung.
'
'      Beim Endpunkt 1 (IN) fragt der PC Daten an.
'      Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
'      neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
'      Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP2_BUF) abgelegt.
'
'      Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
'      Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
'      Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie
'      viele Bytes gesendet wurden.
'
'      Es wird nach 5 Sekunden Wartezeit einmalig der Text "Hallo" gesendet.
'      Danach werden die 8 Bit (Tasten) von Port B eingelesen (nullaktiv).
'      Wurde eine Taste betaetigt, so wird ein Zeichen gesendet.
'
'      Weiter unten ist die Konstanten Keyboard definiert.
'      Mit ihr kann ausgewaehlt werden ob eine Tastatur emuliert werden soll
'      (Keyboard = 1).
'
'      Weitere Dokumentation auf www.weigu.lu/b/usb
'
'
' Copyright (c) 2013   Jean-Claude FELTES & Guy WEILER
'                          weigu[at]weigu[dot]lu

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' Licence: GPL-3

'Hier bitte Chip ($regfile) und Quarz ($crystal) durch Auskommentieren
'fr das entsprechende Board auswaehlen!!
'Teensy 2.0: ATmega32u4  16 MHz
'AT90USBKEY: AT90USB1287  8 MHz
'Teensy 1.0: AT90USB162  16 MHz

'$regfile = "m32u4def.dat"               '_CHIP = 75
'$regfile = "usb1287.dat"                '_CHIP = 70
$regfile = "usb162.dat"                 '_CHIP = 74
'Achtung B2.0.7.3 Fehler in Datei USB_EP statt USB_COM!!
'$regfile = "m32u2def.dat"               '_CHIP = 78
$crystal = 16000000
'$crystal = 8000000

$hwstack = 100
$swstack = 100
$framesize = 100

'Interrupts
On Usb_gen Usb_isr_general
On Usb_com Usb_isr_communication
On Oc1a Isrtc1

' Hier kann wahlweise die Computermaus- oder die Tastatur-Emulation mit 1
' freigeschaltet werden
Const Mouse = 0                         'Mouse = 0 keine Mausemulation
Const Keyboard = 1                      'Keyboard = 0 keine Tastaturemulation

Goto Init                               'Leider noetig damit globale Variablen eingebunden werden
                                         'ohne dass der Code der Isr ausgefhrt wird
$include "/lib/usb_lib_hid_v1_2.bas"    'USB-Bibliothek einbinden
Init:

Dim Blinkflag As Bit                    'Blinken ein/aus
Dim Btmp1 As Byte                       'Zwischenregister
Dim Adcval As Word

Declare Sub Sndkey(byval Modi As Byte , Byval Code As Byte)

'Ein- und Ausgaenge
Config Pind.6 = Output                  'blinkende LED (onboard)
Config Portb = Input                    '8 Tasten
Portb = &HFF                            'Pull-Up

Config Clockdiv = 1

'Config Timer1 = Timer
Tccr1a = 0
Tccr1b = &H0D
#if _chip = 70                          'At90usb1287
Ocr1a = 3905
#else
Ocr1a = 7811
#endif
Enable Oc1a                             'Timer Interrupt erlauben


Gosub Usb_init_device                   'USB-Aktivierung

Wait 5                                  'Warte 5 Sekunden

'Schreibe "Hallo"
 Call Sndkey(&H02 , &H0B)              'H(shift + H)
 Call Sndkey(&H00 , &H04)              'a
 Call Sndkey(&H00 , &H0F)              'l
 Call Sndkey(&H00 , &H0F)              'l
 Call Sndkey(&H00 , &H12)              'o
 Call Sndkey(&H00 , &H2C)              'Space

'-------------------------------------------------------------------------------

Do
'8 Tasten abfragen
 If Pinb.0 = 0 Then Call Sndkey(&H00 , &H14)       'q
 If Pinb.1 = 0 Then Call Sndkey(&H00 , &H1A)       'w
 If Pinb.2 = 0 Then Call Sndkey(&H00 , &H08)       'e
 If Pinb.3 = 0 Then Call Sndkey(&H00 , &H15)       'r
 If Pinb.4 = 0 Then Call Sndkey(&H00 , &H17)       't
 If Pinb.5 = 0 Then Call Sndkey(&H00 , &H1C)       'z
 If Pinb.6 = 0 Then Call Sndkey(&H00 , &H18)       'u
 If Pinb.7 = 0 Then Call Sndkey(&H00 , &H0C)       'i
 Waitms 100
Loop

'Interuptroutine Timer1 Compare Match A Interrupt
'Toggelt LED an PD6
Isrtc1:
Toggle Portd.6
Return

'Unterprogramm zum Senden eines Zeichens
'Parameter modi:  byte for modifier key, Bit 7-0:
'                 R GUI, R ALT, R SHIFT, R CTRL,L GUI, L ALT, L SHIFT, L CTRL
'          code: scan code from key  */
Sub Sndkey(modi As Byte , Code As Byte)
  Ep1_cnt = 8                           '8 bytes (for windows!!)
  Disable Interrupts
  Ep1_buf(1) = Modi                     'modifier byte
  Ep1_buf(2) = 0                        'reserved byte = 0
  Ep1_buf(3) = Code                     'scan code (up to 6 keys)
  Ep1_buf(4) = 0
  Ep1_buf(5) = 0
  Ep1_buf(6) = 0
  Ep1_buf(7) = 0
  Ep1_buf(8) = 0
  Enable Interrupts
  Ep1_flag = 1                          'set EP1 flag
  Waitms 5                              'higher than bInterval in Endpoint Descriptor!
  Ep1_cnt = 8                           '8 bytes
  Disable Interrupts
  Ep1_buf(1) = 0                        'modifier byte
  Ep1_buf(2) = 0                        'reserved byte = 0
  Ep1_buf(3) = 0                        'stop pressing key
  Ep1_buf(4) = 0
  Ep1_buf(5) = 0
  Ep1_buf(6) = 0
  Ep1_buf(7) = 0
  Ep1_buf(8) = 0
  Enable Interrupts
  Ep1_flag = 1                          'set EP1 flag
  Waitms 5                              'higher than bInterval in Endpoint Descriptor!
End Sub
